import os
import execjs
from fake_useragent import UserAgent

ua = UserAgent()

def get_token():
    '''获取token'''
    with open(os.path.join(os.path.dirname(__file__), 'hexin-v.js'), 'r') as f:
        jscontent = f.read()
    context = execjs.compile(jscontent)
    return context.call("v")

def headers(cookie=None):
    return {
        'hexin-v': get_token(),
        'User-Agent': ua.random,
        'cookie': cookie
    }



import pandas as pd
import json
import pydash as _
import requests as rq
from .headers import headers

def get_url(url):
    res = rq.request(
        method='GET',
        url=f'http://www.iwencai.com{url}',
        headers=headers()
    )
    result = json.loads(res.text)
    return result.get('data')

def xuangu_tableV1_handler(comp, comps):
    '''xuangu_tableV1类型'''
    return {
        'condition': _.get(comp, 'data.meta.extra.condition'),
        'comp_id': comp['cid'],
        'uuid': comp['puuid']
    }

def common_handler(comp, comps):
    '''common类型'''
    datas = _.get(comp, 'data.datas')
    if isinstance(datas, list):
        return pd.DataFrame.from_dict(datas)
    else:
        return _.get(comp, 'data')

def container_handler(comp, comps):
    '''container类型'''
    result = {}
    for uuid in _.get(comp, 'config.children', []):
        child = _.find(comps, lambda c: c.get('uuid') == uuid)
        key = _.get(child, 'show_type')
        if key is not None and key != '':
            result[key] = show_type_handler(child, comps)
    return result

def txt_handler(comp, comps):
    '''txt类型'''
    content = _.get(comp, 'data.content')
    return content

def tab4_handler(comp, comps):
    '''tab4类型'''
    result = {}
    for tab in comp.get('tab_list'):
        tab_name = tab.get('tab_name')
        tab_list = tab.get('list')
        if tab_name is not None:
            tabResult = result[tab_name] = {}
            for tcomp in tab_list:
                show_type = tcomp.get('show_type')
                tabResult[show_type] = show_type_handler(tcomp, tab_list)
    return result

def tab1_handler(comp, comps):
    '''tab1类型'''
    result = {}
    data = comp.get('data')
    for tab in comp.get('tab_list'):
        tab_name = tab.get('tab_name')
        tab_list = tab.get('list')
        if tab_name is not None:
            tabResult = result[tab_name] = {}
            for tcomp in tab_list:
                show_type = tcomp.get('show_type')
                data_index = tcomp.get('data_index')
                tcomp.set('data', data.get(data_index))
                tabResult[show_type] = show_type_handler(tcomp, tab_list)
    return result

def dragon_tiger_stock_handler(comp, comps):
    '''龙虎榜分析'''
    result ={}
    data = _.get(comp, 'data.datas.0')
    detail = data.pop('detail', None)
    
    result['data'] = pd.DataFrame.from_dict([data])
    if detail is not None:
        result['detail'] = {
            'buy': pd.DataFrame.from_dict(_.get(detail[0], 'buy.datas')),
            'sell': pd.DataFrame.from_dict(_.get(detail[0], 'sell.datas'))
        }
    return result

def wiki1_handler(comp, comps):
    url = _.get(comp, 'data.url')
    if url is not None:
        wcomp = get_url(url)
        if wcomp is not None:
            return show_type_handler(wcomp, comps)
        else:
            return None
    return None

def textblocklinkone_handler(comp, comps):
    data = _.get(comp, 'data.result.data')
    return pd.DataFrame.from_dict(data)

def nestedblocks_handler(comp, comps):
    '''股东户数分析'''
    subBlocks = _.get(comp, 'data.result.subBlocks.0.subBlocks')
    result = []
    for sub in subBlocks:
        url = sub.get('url')
        sub_comp = get_url(url)
        if sub_comp is not None:
            result.append(show_type_handler(sub_comp, comps))
    return result




show_type_handler_dict = {
    'container': container_handler,
    'txt1': txt_handler,
    'txt2': txt_handler,
    'tab4': tab4_handler,
    'dragon_tiger_stock': dragon_tiger_stock_handler,
    'tab1': tab1_handler,
    # 'wiki1': wiki1_handler,
    'textblocklinkone': textblocklinkone_handler,
    'nestedblocks': nestedblocks_handler
}

def show_type_handler(comp, comps):
    '''处理每种不同的show_type类型'''
    show_type = comp.get('show_type')
    handler = show_type_handler_dict.get(show_type)
    if handler is not None:
        return handler(comp, comps)
    else:
        return common_handler(comp, comps)

def get_key(comp):
    '''获取每一项的key'''
    h1 = _.get(comp, 'title_config.data.h1') or _.get(comp, 'config.title') or _.get(comp, 'show_type')
    return h1

def multi_show_type_handler(components):
    '''处理多个show_type类型的数据'''
    result = {}
    for comp in components:
        key = get_key(comp)
        value = show_type_handler(comp, components)
        if key is not None and key != '' and value is not None:
            result[key] = value

    return result


def convert(res):
    '''处理get_robot_data的结果'''
    result = json.loads(res.text)
    content = _.get(result, 'data.answer.0.txt.0.content')
    if type(content) == str:
        content = json.loads(content)
    components = content['components'] 
    params = {}
    if (len(components) == 1 and _.get(components[0], 'show_type') == 'xuangu_tableV1'):
        params = {
            'data': xuangu_tableV1_handler(components[0], components),
            'row_count' : _.get(components[0], 'data.meta.extra.row_count')
        }
    else:
        params = {
            'data': multi_show_type_handler(components)
        }
    return params



import json
from typing import List
import math

import requests as rq
import pandas as pd
import time
import logging
import pydash as _
from .convert import convert
from .headers import headers

handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('[pywencai] %(asctime)s - %(levelname)s - %(message)s'))
logger = logging.getLogger(__name__)
logger.addHandler(handler)
logger.setLevel(logging.INFO)


def while_do(do, retry=10, sleep=0, log=False):
    count = 0
    while count < retry:
        time.sleep(sleep)
        try:
            return do()
        except:
            log and logger.warning(f'{count+1}次尝试失败')
            count += 1
    return None


def get_robot_data(**kwargs):
    '''获取condition'''
    retry = kwargs.get('retry', 10)
    sleep = kwargs.get('sleep', 0)
    question = kwargs.get('query')
    log = kwargs.get('log', False)
    query_type = kwargs.get('query_type', 'stock')
    cookie = kwargs.get('cookie', None)
    request_params = kwargs.get('request_params', {})
    data = {
        'perpage': 10,
        'page': 1,
        'source': 'Ths_iwencai_Xuangu',
        'secondary_intent': query_type,
        'question': question
    }

    pro = kwargs.get('pro', False)

    if pro:
        data['iwcpro'] = 1

    log and logger.info(f'获取condition开始')

    def do():
        res = rq.request(
            method='POST',
            url='http://www.iwencai.com/customized/chart/get-robot-data',
            json=data,
            headers=headers(cookie),
            **request_params
        )
        params = convert(res)
        log and logger.info(f'获取get_robot_data成功')
        return params

    result = while_do(do, retry, sleep, log)

    if result is None:
        log and logger.info(f'获取get_robot_data失败')

    return result


def replace_key(key):
    '''替换key'''
    key_map = {
        'question': 'query',
        'sort_key': 'urp_sort_index',
        'sort_order': 'urp_sort_way'
    }
    return key_map.get(key, key)


def get_page(**kwargs):
    '''获取每页数据'''
    retry = kwargs.pop('retry', 10)
    sleep = kwargs.pop('sleep', 0)
    log = kwargs.pop('log', False)
    cookie = kwargs.pop('cookie', None)
    find = kwargs.pop('find', None)
    query_type = kwargs.get('query_type', 'stock')
    request_params = kwargs.get('request_params', {})
    pro = kwargs.get('pro', False)
    if find is None:
        data = {
            'perpage': 100,
            'page': 1,
            'source': 'Ths_iwencai_Xuangu',
            **kwargs
        }
        target_url = 'http://www.iwencai.com/gateway/urp/v7/landing/getDataList'
        if pro:
            target_url = f'{target_url}?iwcpro=1'
        path = 'answer.components.0.data.datas'
    else:
        if isinstance(find, List):
            # 传入股票代码列表时，拼接
            find = ','.join(find)
        data = {
            'perpage': 100,
            'page': 1,
            'source': 'Ths_iwencai_Xuangu',
            'query_type': query_type,
            'question': find,
            **kwargs
        }
        target_url = 'http://www.iwencai.com/unifiedwap/unified-wap/v2/stock-pick/find'
        path = 'data.data.datas'
    
    log and logger.info(f'第{data.get("page")}页开始')

    def do():
        res = rq.request(
            method='POST',
            url=target_url,
            data=data,
            headers=headers(cookie),
            timeout=(5, 10),
            **request_params
        )
        result_do = json.loads(res.text)
        data_list = _.get(result_do, path)
        # print(len(data_list))
        if len(data_list) == 0:
            log and logger.error(f'第{data.get("page")}页返回空！')
            raise Exception("data_list is empty!")
        log and logger.info(f'第{data.get("page")}页成功')
        return pd.DataFrame.from_dict(data_list)
    
    result = while_do(do, retry, sleep, log)

    if result is None:
        log and logger.error(f'第{data.get("page")}页失败')

    return result


def can_loop(loop, count):
    return count < loop


def loop_page(loop, row_count, **kwargs):
    '''循环分页'''
    count = 0
    perpage = kwargs.pop('perpage', 100)
    max_page = math.ceil(row_count / perpage)
    result = None
    if 'page' not in kwargs:
        kwargs['page'] = 1
    initPage = kwargs['page']
    loop_count = max_page if loop is True else loop
    while can_loop(loop_count, count):
        kwargs['page'] = initPage + count
        resultPage = get_page(**kwargs)
        count = count + 1
        if result is None:
            result = resultPage
        else:
            result = pd.concat([result, resultPage], ignore_index=True)

    return result


def get(loop=False, **kwargs):
    '''获取结果'''
    kwargs = {replace_key(key): value for key, value in kwargs.items()}
    params = get_robot_data(**kwargs)
    data = params.get('data')
    condition = _.get(data, 'condition')
    if condition is not None:
        kwargs = {**kwargs, **data}
        find = kwargs.get('find', None)
        if loop and find is None:
            row_count = params.get('row_count')
            return loop_page(loop, row_count, **kwargs)
        else:
            return get_page(**kwargs)
    else:
        no_detail = kwargs.get('no_detail')
        if no_detail != True:
            return data
        else:
            return None